import re
import sys
import csv
import math
import json
import codecs
import time
import datetime
import operator
import itertools
import numpy as np
import matplotlib.pyplot as plt

def time_map():
	tdict = dict()
	for line in open('data/submit-times').readlines():
		aid, start, end = line.split('\t')
		start_t = datetime.datetime.strptime(start.strip(), "%Y-%m-%d %H:%M:%S")
		end_t = datetime.datetime.strptime(end.strip(), "%Y-%m-%d %H:%M:%S")
		tdict[aid] = (start_t, end_t)
	return tdict

def format_for_graph(data):
	ret = list()
	mindate = None
	for l in data.keys(): 
		lang = data[l]	
		begin = lang[0]
		if(mindate == None or begin < mindate):
			mindate = begin
	for l in data.keys():
		lang = data[l]	
		complete = lang[1] - lang[0]
		start = lang[0] - mindate
		ret.append((l,complete, start, start+complete))
	return sorted(ret, key=operator.itemgetter(3)) 

def format_for_time_series(data):
	hitlangs = {row['assign'] : row['lang'] for row in csv.DictReader(open('extracted_data/quality.assigns.txt'), delimiter='\t')}
	all_times = dict()
	for aid, complete, start, total in data:	
		try : lang = hitlangs[aid]
		except : continue
		if lang not in all_times:
			all_times[lang] = list()
		if lang == 'ru' or lang == 'ur': 
			all_times[lang].append(total+datetime.timedelta(days=2))
		else:
			all_times[lang].append(total)
	try:
		all_times.pop('en')
	except KeyError:
		pass
	return all_times

def time_series(data, num=40):
	SEC_PER_DAY = 86400
	x = list()
	y = list()
	names = list()
	sort_order = [t[0] for t in sorted(data.iteritems(), key=lambda t : max(t[1]))]
	for i,lang in enumerate(sort_order[:num]):
		names.append(lang)
		x += [t.total_seconds() for t in data[lang]]
		y += [5+(i*5)]*len(data[lang])
	plt.scatter(x,y,marker='|', alpha=0.1)
	plt.xlim([min(x),max(x)+5])
	plt.ylim([0,max(y)+5])
        plt.yticks(sorted(list(set(y))), tuple(names))
	xmax = int(math.ceil(max(x)))
	xmin = int(math.ceil(min(x)))
	max_days = int(math.ceil(max(x)/SEC_PER_DAY))
        plt.xticks(range(xmin,xmax,5*SEC_PER_DAY), range(0,max_days,5))
        plt.xlabel('Time in days')
	plt.show()

if __name__ == '__main__':

	time_series(format_for_time_series(format_for_graph(time_map())))

